import re

class DialogExtractor:
    def dialog_turns(self, text: str):
        '''Extract dialog turns from input text using regex.'''
        # Remove any HTML tags
        text = re.sub('<.*?>', '', text, flags=re.DOTALL)
        # Regular expression to capture speaker and their corresponding text
        return re.findall(r'([A-Za-z0-9 ]+)\s*:\s*((?:.|\n)*?)(?=\n[A-Za-z0-9 ]+\s*:\s*|\Z)', text)

# Example usage
text = """
Speaker 1 : There is at least one apple. Therefore there are some apples.
Speaker 2 : There is at least one apple2. Therefore there are some apples3.
Speaker 2 : There is at least one apple4. Therefore there are some apples5.
"""

extractor = DialogExtractor()
turns = extractor.dialog_turns(text)

for turn in turns:
    print(f'Name: {turn[0]}')
    print(f'Text: {turn[1].strip()}\n')



from typing import Dict, List
from collections import defaultdict
import traceback
from flask import jsonify

# Decorator for error handling
def handle_errors(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except KeyError:
            return jsonify({'error': 'File not found in request'}), 400
        except Exception as e:
            traceback.print_exc()
            return jsonify({'error': str(e)}), 500
    return wrapper
    
def top_freq_list(xs, top):
    counts = defaultdict(int)
    for x in xs:
        counts[x] += 1
    return sorted(counts.items(), reverse=True, key=lambda tup: tup[1])[:top]

def get_file(file_obj):
    f_name = file_obj.filename    
    file_obj.save(f_name)
    file = open(f_name,'r')
    return f_name

  
def frequent_tuple(tuples):
    count_tuple={}
    for tup in tuples:
        length=len(tup[0].split(" "))
        if length in count_tuple.keys():
            count_tuple[length]+=1
        else:
            count_tuple[length]=1
    sorted_dct=dict(sorted(count_tuple.items(), reverse=True,key=lambda item: item[1]))
    return next(iter( sorted_dct.items() ))[0] 
    
def identyfy_maxs_index(x,bar): 
    return x > bar










import logging
from src.utility import get_next_max_id
import json

from src.templates import PropositionalizerOutput




import logging
from src.data import Data
from xaif_eval import xaif

logging.basicConfig(datefmt='%H:%M:%S',
                    level=logging.DEBUG)



class Propositionalizer():
	def __init__(self,file_obj):
		self.file_obj = file_obj
		self.f_name = file_obj.filename
		self.file_obj.save(self.f_name)
		file = open(self.f_name,'r')


	def is_valid_json(self):
		''' check if the file is valid json
		'''

		try:
			json.loads(open(self.f_name).read())
		except ValueError as e:			
			return False

		return True
	def is_valid_json_aif(self,aif_nodes):
		if 'nodes' in aif_nodes and 'locutions' in aif_nodes and 'edges' in aif_nodes:
			return True
		return False
		

	def get_aif(self, format='xAIF'):

		with open(self.f_name) as file:
			data = file.read()
			x_aif = json.loads(data)
			if format == "xAIF":
				return x_aif
			else:
				aif = x_aif.get('AIF')
				return json.dumps(aif)


	def propositionalizer(self,):
		xAIF_input = self.get_aif()
		logging.info(f"xAIF data:  {xAIF_input}, {self.file_obj}")  
		xaif_obj = xaif.AIF(xAIF_input)
		is_json_file = self.is_valid_json()
		if is_json_file:				
			json_dict = xaif_obj.aif
			if self.is_valid_json_aif(json_dict):
				nodes = json_dict['nodes']		
				nodes, edges  = json_dict['nodes'], json_dict['edges']				
				original_nodes = nodes.copy()			
				i_nodes_lis = []
				for nodes_entry in original_nodes:
					propositions = nodes_entry['text']
					node_id = nodes_entry['nodeID'] 
					if propositions not in i_nodes_lis:
						if nodes_entry['type'] == "L":						
							inode_id = get_next_max_id(nodes, "nodeID")
							nodes.append({'text': propositions, 'type':'I','nodeID': inode_id})
							i_nodes_lis.append(propositions)
							y_id = get_next_max_id(nodes, "nodeID")
							nodes.append({'text': 'Default Illocuting', 'type':'YA','nodeID': y_id})
							if edges:	
								edge_id = get_next_max_id(edges, "edgeID")
							else:
								edge_id = 0
							edges.append({'toID': y_id, 'fromID':node_id,'edgeID': edge_id})
							edge_id = get_next_max_id(edges, "edgeID")
							edges.append({'toID': inode_id, 'fromID':y_id,'edgeID': edge_id})

				return xaif_obj.xaif
			else:
				return("Incorrect json-aif format")
		else:
			return("Incorrect input format")



	####################
	def propositionalizer_default(self,):	
		json_aif = self.propositionalizer() 	
		return json_aif





import re
from flask import json
import logging
# Explicitly import JSONDecodeError for better clarity
from json.decoder import JSONDecodeError
from xaif_eval import xaif

logging.basicConfig(datefmt='%H:%M:%S', level=logging.DEBUG)

from src.data import AIF
from src.templates import TurninatorOutput






class Turninator():
    def __init__(self,file_obj):
        self.file_obj = file_obj
        self.f_name = file_obj.filename
        self.file_obj.save(self.f_name)
        file = open(self.f_name,'r')
    
    def dialog_turns_old(self, text: str):
        '''Extract dialog turns from input text using regex.'''
        text = re.sub('<.*?>', '', text, flags=re.DOTALL)
        return re.findall(r'([A-Za-z ]+):((?:.|\n)*?)(?=(?:^[A-Za-z ]+:)|\Z)', text)
    
    def dialog_turns(self, text: str):
            '''Extract dialog turns from input text using regex.'''
            # Remove any HTML tags
            text = re.sub('<.*?>', '', text, flags=re.DOTALL)
            # Regular expression to capture speaker and their corresponding text
            return re.findall(r'([A-Za-z0-9_ -]+)\s*:\s*((?:.|\n)*?)(?=\n[A-Za-z0-9_ -]+\s*:\s*|\Z)', text)

    def monolog_text(self, text: str) -> str:
        '''Extract the entire text if monolog.'''
        return re.sub('<.*?>', '', text, flags=re.DOTALL)
    
    def is_valid_json(self):
        ''' check if the file is valid json
        '''

        try:
            json.loads(open(self.f_name).read())
        except ValueError as e:			
            return False

        return True
    def is_valid_json_aif(self,aif_nodes):
        if 'nodes' in aif_nodes and 'locutions' in aif_nodes and 'edges' in aif_nodes:
            return True
        return False
    ###

    def get_aif(self, format='xAIF'):
        try:
            # Open and read the file content
            with open(self.f_name, 'r') as file:
                data = file.read()  # Read the raw content
                
                print(f"Raw data read from file: {data}")  # Debugging step
                
                # Convert to Python dictionary (or JSON object)
                x_aif = json.loads(data)  # Parse JSON string
                
                # If 'AIF' exists and is a string, convert it to a dictionary
                if 'AIF' in x_aif and isinstance(x_aif['AIF'], str):
                    try:
                        x_aif['AIF'] = json.loads(x_aif['AIF'])  # Parse 'AIF' value
                    except json.JSONDecodeError:
                        print("Warning: 'AIF' value is not valid JSON.")
                
                print(f"Parsed JSON object with 'AIF' as JSON: {x_aif}")  # Debugging step
                
                # Return the modified dictionary
                return x_aif
        
        except json.JSONDecodeError as e:
            # Handle JSON errors gracefully
            print(f"Error decoding JSON: {e}")
            return None
        except Exception as e:
            # Handle other exceptions
            print(f"An unexpected error occurred: {e}")
            return None


    def turninator_default(self,):
        # Get the file path from the path object

        AIF_obj = AIF()

        extended_json_aif = {}
        if self.f_name.endswith("json"):

            xAIF_input = self.get_aif()
            #logging.info(f"xAIF data:  {xAIF_input}, {self.file_obj}")  
            xaif_obj = xaif.AIF(xAIF_input)
            
            is_json_file = self.is_valid_json()
            
            if is_json_file:				
                nodes, edges, locutions = [], [], []
                extended_json_aif = xaif_obj.xaif
                logging.info(f"extended_json_aif:  {extended_json_aif}")  
                if 'AIF' in extended_json_aif and 'text' in extended_json_aif:
                    json_dict = extended_json_aif['AIF'] 
                    dialog = extended_json_aif.get('dialog', False)
                    OVA = extended_json_aif.get('OVA', [])
                    # Handle the case where 'json_dict' is a string
                    if isinstance(json_dict, str):
                        if json_dict.startswith('"'):
                            json_dict = json_dict.replace("\"", "")
                            json_dict = dict(json_dict)
                            logging.info(f'processing monolog text in json extenssion')
                    if not isinstance(json_dict, dict):
                        json_dict = json.loads(json_dict)
                    # Extract values associated with specific keys from the AIF section
                    schemefulfillments, descriptorfulfillments = AIF_obj.get_xAIF_arrays(aif_section=json_dict,
                                                                                     xaif_elements=['schemefulfillments', 'descriptorfulfillments'])
                    logging.info(f"xAIF valid:  {is_json_file}")  
                    participants = json_dict.get("participants", [])
                    if isinstance(extended_json_aif['text'], dict):
                        text = extended_json_aif['text']['txt']
                        logging.info(f'text with dict')
                    else:
                        text = extended_json_aif['text']  # gets the text
                        text = text + "\n"
                        logging.info(f'text without dict')
                        #logging.info(f'text :{text}')
                    if isinstance(text, str):
                        json_object = json.dumps(text)
                        json_object = json.loads(json_object) 
                        logging.info(f'text dumped')                   
                    is_dialog = extended_json_aif.get('dialog')
                    text_with_span = ""
                    node_id, person_id = 0, 0
                    # Extract dialog turns if it's a dialog, otherwise extract monolog text
                    logging.info(f'text : {text}')
                    logging.info(f'is dialog : {is_dialog}')
                    speakers_and_turns = self.dialog_turns(text) if is_dialog and len(self.dialog_turns(text)) else self.monolog_text(text)
                    #logging.info(f'speakers_and_turns : {speakers_and_turns}')
                    logging.info(f'length if turns ====================== : {len(self.dialog_turns(text)) }')
                    if is_dialog and len(self.dialog_turns(text)):
                        logging.info(f'processing dialog text from text format')
                        speakers_and_turns = self.dialog_turns(text)
                        nodes, locutions, participants, text_with_span, node_id, person_id = AIF_obj.create_turn_entry(
                            nodes, node_id, person_id, text_with_span, speakers_and_turns, locutions, participants, is_dialog)
                    else:
                        logging.info(f'====== Something else')
                        if not is_dialog:
                            logging.info(f'processing monolog text since not dialog')
                            speakers_and_turns = self.monolog_text(text)
                            nodes, locutions, participants, text_with_span, node_id, person_id = AIF_obj.create_turn_entry(
                                nodes, node_id, person_id, text_with_span, speakers_and_turns, locutions, participants, is_dialog)
                    return TurninatorOutput.format_output(nodes, edges, locutions, 
                                                          schemefulfillments, descriptorfulfillments, participants, 
                                                          OVA, text_with_span, dialog, json_dict, extended_json_aif)
                else:
                    if 'text' in extended_json_aif:
                        node_id, person_id = 0, 0
                        if isinstance(extended_json_aif['text'], dict):
                            text = extended_json_aif['text']['txt']
                        else:
                            text = extended_json_aif['text'] + "\n"
                        
                        aif, json_aif, OVA = {}, {}, {}      
                        text_with_span = ""
                        nodes, edges, schemefulfillments, descriptorfulfillments, participants, locutions = [], [], [], [], [], []
                        speakers_and_turns = self.monolog_text(text)                   
                        nodes, locutions, participants, text_with_span, node_id, person_id = AIF_obj.create_turn_entry(
                            nodes, node_id, person_id, text_with_span, speakers_and_turns, locutions, participants, False)
                        return TurninatorOutput.format_output(nodes, edges, locutions, schemefulfillments, descriptorfulfillments, participants, OVA, text_with_span, aif, extended_json_aif)
            else:
                logging.info(f'the file is not in a correct json format')
                return "Invalid json"
        else:
            # Non-json data is treated as monolog
            node_id, person_id = 0, 0
            with open(self.f_name, 'r') as file:
                data = file.read() + "\n"              
            aif, json_aif, OVA = {}, {}, {}       
            text_with_span = ""
            nodes, edges, schemefulfillments, descriptorfulfillments, participants, locutions = [], [], [], [], [], []
            speakers_and_turns = self.monolog_text(data)                    
            nodes, locutions, participants, text_with_span, node_id, person_id = AIF_obj.create_turn_entry(
                nodes, node_id, person_id, text_with_span, speakers_and_turns, locutions, participants, False)
            return TurninatorOutput.format_output(nodes, edges, locutions, schemefulfillments, descriptorfulfillments, participants, OVA, text_with_span,aif, json_aif)






from flask import Flask, request,render_template_string
from prometheus_flask_exporter import PrometheusMetrics
import markdown2
from flask_cors import CORS

from src.segmenter import Segmenter
from src.data import Data
from src.utility import handle_errors

import logging
logging.basicConfig(datefmt='%H:%M:%S',
                    level=logging.DEBUG)

app = Flask(__name__)



CORS(app, resources={r"/*": {"origins": "https://arg-tech.github.io"}})
metrics = PrometheusMetrics(app)
	
@app.route('/segmenter-01', methods = ['GET', 'POST'])
@metrics.summary('requests_by_status', 'Request latencies by status',
                 labels={'status': lambda r: r.status_code})
@metrics.histogram('requests_by_status_and_path', 'Request latencies by status and path',
                   labels={'status': lambda r: r.status_code, 'path': lambda: request.path})
@handle_errors  
def segmenter_defult():
	if request.method == 'POST':
		file_obj = request.files['file']
		segmenter = Segmenter(file_obj)
		result=segmenter.segmenter_default()
		return result
	if request.method == 'GET':
		# Read the markdown file
		with open('README.md', 'r') as file:
			md_content = file.read()

		# Convert to HTML
		html_content = markdown2.markdown(md_content)

		# Add CSS link
		css_link = '<link rel="stylesheet" href="https://example.com/path/to/your/styles.css">'
		html_with_css = f"<html><head>{css_link}</head><body>{html_content}</body></html>"

		# Render the HTML content as a template
		return render_template_string(html_with_css)

	
if __name__ == "__main__":
    app.run(host="0.0.0.0", port=int("5005"), debug=False)	  



from typing import Dict, List
from collections import defaultdict
import traceback
from flask import jsonify

def handle_errors(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except KeyError:
            return jsonify({'error': 'File not found in request'}), 400
        except Exception as e:
            traceback.print_exc()
            return jsonify({'error': str(e)}), 500
    return wrapper

    
def top_freq_list(xs, top):
    counts = defaultdict(int)
    for x in xs:
        counts[x] += 1
    return sorted(counts.items(), reverse=True, key=lambda tup: tup[1])[:top]

def get_file(file_obj):
    f_name = file_obj.filename    
    file_obj.save(f_name)
    file = open(f_name,'r')
    return f_name

  
def frequent_tuple(tuples):
    count_tuple={}
    for tup in tuples:
        #print('tup..............................................',tup)
        length=len(tup[0].split(" "))
        if length in count_tuple.keys():
            count_tuple[length]+=1
        else:
            count_tuple[length]=1
    sorted_dct=dict(sorted(count_tuple.items(), reverse=True,key=lambda item: item[1]))
    return next(iter( sorted_dct.items() ))[0] 
    
    

def identyfy_maxs_index(x,bar): 
    return x > bar

def get_next_max_id(nodes, n_type):
    """
    This function takes a list of nodes and returns the maximum node ID.

    Arguments:
    - nodes (List[Dict]): a list of nodes, where each node is a dictionary containing a node ID

    Returns:
    - (int): the maximum node ID in the list of nodes
    """
    # Initialize a variable to store the maximum node ID found so far
    max_id  = 0
    lef_n_id, right_n_id = 0, ""

    if isinstance(nodes[0][n_type],str):
        if "_" in nodes[0][n_type]:
            
            #logging.debug('with hyphen')       
            # Loop through each node in the list of nodes
            for node in nodes:
                # Check if the current node ID is greater than the current maximum
                #logging.debug(node)
                temp_id = node[n_type]
                if "_" in temp_id:
                    nodeid_parsed = temp_id.split("_")
                    lef_n_id, right_n_id = int(nodeid_parsed[0]), nodeid_parsed[1]
                    if lef_n_id > max_id:
                        max_id = lef_n_id
            #logging.debug(str(int(max_id)+1)+"_"+str(right_n_id))
            return str(int(max_id)+1)+"_"+str(right_n_id)
        else:
            for node in nodes:
                # Check if the current node ID is greater than the current maximum
                temp_id = int(node[n_type])     
                if temp_id > max_id:
                    # If it is, update the maximum to the current node ID
                    max_id = temp_id   
            # Return the maximum node ID found
            return str(max_id+1)

    elif isinstance(nodes[0][n_type],int):	
        for node in nodes:
            # Check if the current node ID is greater than the current maximum
            temp_id = node[n_type]     
            if temp_id > max_id:
                # If it is, update the maximum to the current node ID
                max_id = temp_id   
        # Return the maximum node ID found
        return max_id+1

def get_speaker(
    nodeID_speaker,
    node_id: int, 
    locutions: List[Dict[str, int]], 
    participants: List[Dict[str, str]]
    ) -> str:
    """
    This function takes a node ID, a list of locutions, and a list of participants, and returns the name of the participant who spoke the locution with the given node ID, or "None" if the node ID is not found.

    Arguments:
    - node_id (int): the node ID to search for
    - locutions (List[Dict]): a list of locutions, where each locution is a dictionary containing a node ID and a person ID
    - participants (List[Dict]): a list of participants, where each participant is a dictionary containing a participant ID, a first name, and a last name

    Returns:
    - (str): the name of the participant who spoke the locution with the given node ID, or "None" if the node ID is not found
    """

    
    # Loop through each locution and extract the person ID and node ID
    for locution in locutions:
        personID = locution['personID']
        nodeID = locution['nodeID']
        
        # Loop through each participant and check if their participant ID matches the person ID from the locution
        for participant in participants:
            if participant["participantID"] == personID:
                # If there is a match, add the participant's name to the nodeID_speaker dictionary with the node ID as the key
                firstname = participant["firstname"]
                surname = participant["surname"]
                nodeID_speaker[nodeID] = (firstname+" "+surname,personID)
                
    # Check if the given node ID is in the nodeID_speaker dictionary and return the corresponding speaker name, or "None" if the node ID is not found
    if node_id in nodeID_speaker:
        return nodeID_speaker[node_id]
    else:
        return ("None None","None")

def get_inode(edges, n_id):
    for entry in edges:
        if n_id == entry['fromID']:
            ya_node_id = entry['toID']
            for entry2 in edges:
                if ya_node_id == entry2['fromID']:
                    inode_id = entry2['toID']
                    return(inode_id, ya_node_id)
    return None, None


from flask import Flask, request
from prometheus_flask_exporter import PrometheusMetrics
from src.data import Data, AIF
from src.propositionalizer import  Propositionalizer 
from src.utility import get_file,handle_errors
import logging
logging.basicConfig(datefmt='%H:%M:%S',
                    level=logging.DEBUG)



app = Flask(__name__)
metrics = PrometheusMetrics(app)
@app.route('/propositionUnitizer-01', methods = ['GET', 'POST'])
@metrics.summary('requests_by_status', 'Request latencies by status',
                 labels={'status': lambda r: r.status_code})
@metrics.histogram('requests_by_status_and_path', 'Request latencies by status and path',
                   labels={'status': lambda r: r.status_code, 'path': lambda: request.path})
@handle_errors
def propositionalizer_defult():
	if request.method == 'POST':
		file_obj = request.files['file']
		propositionaliser = Propositionalizer(file_obj)
		result=propositionaliser.propositionalizer_default()
		return result
	if request.method == 'GET':
		info = """PropositionUnitizer is an AMF component that performs preprocessing steps on propositions. 
		These steps include converting reported speeches and questions, resolving co-references, and creating 'I' nodes for every 'L' node. 
		This implementation serves as the default PropositionUnitizer, which injects 'I' nodes for every 'L' node. It accepts input and returns xIAF format. 
		The component can be connected to Turninator to inject 'I' nodes into the output of Turninator, as most AMF components process the 'I' nodes."""
		return info
	
if __name__ == "__main__":
    app.run(host="0.0.0.0", port=int("5004"), debug=False)	  




from transformers import BartTokenizer


from transformers import pipeline
from amf_fast_inference import model
import logging
import json

logging.basicConfig(datefmt='%H:%M:%S', level=logging.DEBUG)
#

class Model:
    def __init__(self, model_path):
        self.model_path = model_path
        self.tokenizer = BartTokenizer.from_pretrained(model_path)
        #tokenizer = BertTokenizer.from_pretrained("bert-base-uncased")
        #self.model = BartForSequenceClassification.from_pretrained(model_path)
        loader = model.ModelLoader(model_path)
        pruned_model = loader.load_model()
        self.pipe = pipeline("text-classification", model=pruned_model, tokenizer=self.tokenizer )
        self.RA_TRESHOLD = 80
        self.CA_TRESHOLD = 10

    def predict(self, proposition_pair):
        #proposition1, proposition2 = proposition_pair
        outputs = self.pipe(proposition_pair)
        logging.info(f"xAIF data:  {outputs}") 
        relations = [output['label'] for output in outputs]
        #return self._post_process(proposition1, proposition2)
        return self._post_process(relations)

    def _get_prob(self, text1, text2):
        input_ids = self.tokenizer.encode(text1, text2, return_tensors='pt')
        #logits = self.model(input_ids)[0]
        #[{'label': 'neutral', 'score': 0.796902596950531}]
        logits = self.pipe(text1+ " [SEP] " +text2)
        #logging.info(f"xAIF data:  {logits}")  
        logits = self.pipe(text1+ " [SEP] " +text2)[0]
        entail_contradiction_logits = logits[:, [0, 2]]
        probs = entail_contradiction_logits.softmax(dim=1)
        true_prob = probs[:, 1].item() * 100
        return true_prob

    def _post_process_(self, text1, text2):
        true_prob, arg_rel = 0.0, "None"
        true_prob1 = self._get_prob(text1, text2)
        true_prob2 = self._get_prob(text2, text1)
        true_prob = max(true_prob1, true_prob2)
        
        if true_prob > self.RA_TRESHOLD:
            arg_rel = "RA"
        elif true_prob < self.CA_TRESHOLD:
            arg_rel = "CA"
        else:
            arg_rel = "None"

        return arg_rel
    
    def _post_process(self, results):  
        arg_rels = []  
        arg_rel = "None" 
        for result in results:  
            if result == 'entailment':
                arg_rel = "RA"
            elif result ==  'contradiction':
                arg_rel = "CA"
            else:
                arg_rel = "None"
            arg_rels.append(arg_rel)

        return arg_rels




import json

class SegmenterOutput:
    @staticmethod
    def format_output(nodes, edges, locutions, aif, x_aif):
        aif['nodes'] =  nodes
        aif['edges'] =  edges
        aif['locutions'] =  locutions
        x_aif['AIF'] = aif
        return json.dumps(x_aif)
